import io
import json
import logging
import os
from pathlib import Path
import shutil
import uuid
from attr import dataclass

from ..objects import db_view_creator
from . import models
import tempfile
from file2txt.converter import get_parser_class
from txt2stix import get_include_path
from txt2stix.stix import txt2stixBundler
from txt2stix.ai_extractor import BaseAIExtractor
from stix2arango.stix2arango import Stix2Arango
from django.conf import settings
from txt2stix.ai_extractor.utils import DescribesIncident


from file2txt.converter import Fanger, get_parser_class
from file2txt.parsers.core import BaseParser
import txt2stix.utils
import txt2stix.txt2stix
import txt2stix.extractions


def all_extractors(names, _all=False):
    retval = {}
    extractors = txt2stix.extractions.parse_extraction_config(
        get_include_path()
    ).values()
    for extractor in extractors:
        if _all or extractor.slug in names:
            retval[extractor.slug] = extractor
    return retval


@dataclass
class ReportProperties:
    name: str = None
    identity: dict = None
    tlp_level: str = None
    confidence: int = None
    labels: list[str] = None
    created: str = None
    kwargs: dict = {}


class StixifyProcessor:
    def __init__(
        self,
        file: io.FileIO,
        profile: models.Profile,
        job_id: uuid.UUID,
        post=None,
        file2txt_mode="html",
        report_id=None,
        base_url=None,
        **kwargs,
    ) -> None:
        self.job_id = str(job_id)
        self.extra_data = dict()
        self.report_id = report_id
        self.profile = profile
        self.collection_name = "stixify"
        self.tmpdir = Path(tempfile.mkdtemp(prefix="stixify-"))
        self.file2txt_mode = file2txt_mode
        self.md_images = []
        self.processed_image_base_url = ""
        self.base_url = base_url
        self.incident: DescribesIncident = None
        self.summary = None

        self.filename = self.tmpdir / Path(file.name).name
        self.filename.write_bytes(file.read())

        self.task_name = f"{self.profile.name}/{job_id}/{self.report_id}"

    def setup(self, /, report_prop: ReportProperties, extra={}):
        self.extra_data.update(extra)
        self.report_prop = report_prop

    def file2txt(self):
        parser_class = get_parser_class(self.file2txt_mode, self.filename.name)
        converter: BaseParser = parser_class(
            self.filename,
            self.file2txt_mode,
            self.profile.extract_text_from_image,
            settings.GOOGLE_VISION_API_KEY,
            base_url=self.base_url,
        )
        output = converter.convert(
            processed_image_base_url=self.processed_image_base_url
        )
        if self.profile.defang:
            output = Fanger(output).defang()
        for name, img in converter.images.items():
            img_file = io.BytesIO()
            img_file.name = name
            img.save(img_file, format="png")
            self.md_images.append(img_file)

        self.output_md = output
        self.md_file = self.tmpdir / f"post_md_{self.report_id or 'file'}.md"
        self.md_file.write_text(self.output_md)

    def txt2stix(self):
        extractors = all_extractors(self.profile.extractions)
        extractors_map = {}
        for extractor in extractors.values():
            if extractors_map.get(extractor.type):
                extractors_map[extractor.type][extractor.slug] = extractor
            else:
                extractors_map[extractor.type] = {extractor.slug: extractor}

        self.bundler = txt2stixBundler(
            self.report_prop.name,
            identity=self.report_prop.identity,
            tlp_level=self.report_prop.tlp_level,
            confidence=self.report_prop.confidence,
            labels=self.report_prop.labels,
            description=self.output_md,
            extractors=extractors,
            report_id=self.report_id,
            created=self.report_prop.created,
            **self.report_prop.kwargs,
        )
        self.extra_data["_stixify_report_id"] = str(self.bundler.report.id)
        input_text = txt2stix.utils.remove_links(
            self.output_md,
            self.profile.ignore_image_refs,
            self.profile.ignore_link_refs,
        )
        ai_extractors = [
            txt2stix.txt2stix.parse_model(model_str)
            for model_str in self.profile.ai_settings_extractions
        ]
        self.txt2stix_data = txt2stix.txt2stix.run_txt2stix(
            self.bundler,
            input_text,
            extractors_map,
            ai_content_check_provider=self.profile.ai_content_check_provider
            and txt2stix.txt2stix.parse_model(self.profile.ai_content_check_provider),
            ai_create_attack_flow=self.profile.ai_create_attack_flow,
            ai_create_attack_navigator_layer=self.profile.ai_create_attack_navigator_layer,
            input_token_limit=settings.INPUT_TOKEN_LIMIT,
            ai_settings_extractions=ai_extractors,
            ai_settings_relationships=self.profile.ai_settings_relationships
            and txt2stix.txt2stix.parse_model(self.profile.ai_settings_relationships),
            relationship_mode=self.profile.relationship_mode,
            ignore_extraction_boundary=self.profile.ignore_extraction_boundary,
            ai_extract_if_no_incidence=self.profile.ai_extract_if_no_incidence,
        )
        self.incident = self.txt2stix_data.content_check
        self.summary = self.incident and self.incident.summary
        return self.bundler

    def process(self) -> str:
        logging.info(f"running file2txt on {self.task_name}")
        self.file2txt()
        logging.info(f"running txt2stix on {self.task_name}")
        bundler: txt2stixBundler = self.txt2stix()
        self.write_bundle(bundler)
        logging.info(f"uploading {self.task_name} to arangodb via stix2arango")
        self.upload_to_arango()
        return bundler.report.id

    def write_bundle(self, bundler: txt2stixBundler):
        bundle = json.loads(bundler.to_json())
        self.bundle = json.dumps(bundle, indent=4)
        self.bundle_file = self.tmpdir / f"bundle_{self.report_id}.json"
        self.bundle_file.write_text(self.bundle)

    def upload_to_arango(self):
        s2a = Stix2Arango(
            file=str(self.bundle_file),
            database=settings.ARANGODB_DATABASE,
            collection=self.collection_name,
            stix2arango_note=f"stixifier-report--{self.report_id}",
            ignore_embedded_relationships=self.profile.ignore_embedded_relationships,
            ignore_embedded_relationships_smo=self.profile.ignore_embedded_relationships_smo,
            ignore_embedded_relationships_sro=self.profile.ignore_embedded_relationships_sro,
            host_url=settings.ARANGODB_HOST_URL,
            username=settings.ARANGODB_USERNAME,
            password=settings.ARANGODB_PASSWORD,
        )
        s2a.arangodb_extra_data.update(self.extra_data)
        db_view_creator.link_one_collection(
            s2a.arango.db,
            settings.ARANGODB_DATABASE_VIEW,
            f"{self.collection_name}_edge_collection",
        )
        db_view_creator.link_one_collection(
            s2a.arango.db,
            settings.ARANGODB_DATABASE_VIEW,
            f"{self.collection_name}_vertex_collection",
        )
        s2a.run()

    def __del__(self):
        shutil.rmtree(self.tmpdir)



# Generated by Django 5.1.5 on 2025-05-01 09:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0003_rename_ai_content_check_variable_profile_ai_content_check_provider_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='identity_id',
            field=models.CharField(default=None, max_length=46, null=True),
        ),
    ]



import random
import pytest
from dogesec_commons.objects import conf
from dogesec_commons.objects.helpers import ArangoDBHelper
from tests.objects.data import SRO_DATA
from tests.objects.utils import make_s2a_uploads, request_from_queries
from rest_framework.exceptions import NotFound


def test_get_objects_uses_view_and_has_no_duplicates(subtests):
    helper = ArangoDBHelper(conf.ARANGODB_DATABASE_VIEW, None)

    def test_no_duplicate(response_data, expected_ids):
        objects = response_data["objects"]
        object_ids = {obj["id"] for obj in objects}
        assert (
            len(object_ids) == response_data["page_results_count"]
        ), "duplicates in response"
        assert object_ids == expected_ids, object_ids

    with make_s2a_uploads(
        [
            (
                "collection1",
                [
                    {
                        "type": "ipv4-addr",
                        "spec_version": "2.1",
                        "id": "ipv4-addr--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                        "value": "1.1.1.1",
                    },
                    {
                        "type": "not-an-sco",
                        "spec_version": "2.1",
                        "id": "not-an-sco--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                        "value": "1.1.1.1",
                    },
                    {
                        "type": "marking-definition",
                        "id": "marking-definition--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                    },  # SMO
                    {
                        "type": "extension-definition",
                        "id": "extension-definition--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                    },  # SMO
                    {
                        "type": "network-traffic",
                        "spec_version": "2.1",
                        "id": "network-traffic--4f2724da-746b-5e13-b6e2-30693a9d6977",
                        "dst_ref": "ipv4-addr--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                        "dst_port": 80,
                        "protocols": ["ipv4"],
                    },
                    {
                        "type": "relationship",
                        "id": "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                        "source_ref": "some-source--1",
                        "target_ref": "some-target--1",
                    },  # SRO1
                    {
                        "type": "relationship",
                        "id": "relationship--2c8df0f5-1a2b-4b1b-9f13-21a2767c8f0d",
                        "source_ref": "some-source--1",
                        "target_ref": "some-target--1",
                    },  # SRO3
                    {
                        "type": "vulnerability",
                        "id": "vulnerability--1162f86e-c825-4b20-a69e-ea8a6d9d3948",
                    },  # SDO1
                    {
                        "type": "indicator",
                        "id": "indicator--d28eedd5-8066-4b4a-a6e8-a31795403f97",
                    },  # SDO2
                ],
            ),
            (
                "collection2",
                [
                    {
                        "type": "ipv4-addr",
                        "spec_version": "2.1",
                        "id": "ipv4-addr--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                        "value": "1.1.1.1",
                    },
                    {
                        "type": "ipv4-addr",
                        "spec_version": "2.1",
                        "id": "ipv4-addr--8e594bf5-81f2-5460-9e43-62e6a06794e0",
                        "value": "1.1.1.1/24",
                    },
                    {
                        "type": "marking-definition",
                        "spec_version": "2.1",
                        "id": "marking-definition--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                        "value": "1.1.1.1",
                    },  # SMOs
                    {
                        "type": "language-content",
                        "id": "language-content--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                    },  # SMO
                    {
                        "type": "relationship",
                        "id": "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                        "source_ref": "some-source--2",
                        "target_ref": "some-target--2",
                    },  # SRO1
                    {
                        "type": "relationship",
                        "id": "relationship--1162f86e-c825-4b20-a69e-ea8a6d9d3948",
                        "source_ref": "some-source--3",
                        "target_ref": "some-target--3",
                    },  # SRO2
                    {
                        "type": "vulnerability",
                        "id": "vulnerability--1162f86e-c825-4b20-a69e-ea8a6d9d3948",
                    },  # SDO1
                    {
                        "type": "weakness",
                        "id": "weakness--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                    },  # SDO3
                ],
            ),
        ],
        truncate_collection=True,
    ) as s2a:
        # for filters, expected in subtests_cases:
        with subtests.test("get_scos"):
            response_data = helper.get_scos().data
            test_no_duplicate(
                response_data,
                {
                    "ipv4-addr--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                    "ipv4-addr--8e594bf5-81f2-5460-9e43-62e6a06794e0",
                    "network-traffic--4f2724da-746b-5e13-b6e2-30693a9d6977",
                },
            )

        with subtests.test("get_smos"):
            response_data = helper.get_smos().data
            test_no_duplicate(
                response_data,
                {
                    "language-content--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                    "marking-definition--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                    "extension-definition--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                    ## s2a default objects
                    "marking-definition--613f2e26-407d-48c7-9eca-b8e91df99dc9",
                    "marking-definition--e828b379-4e03-4974-9ac4-e53a884c97c1",
                    "marking-definition--e828b379-4e03-4974-9ac4-e53a884c97c1",
                    "marking-definition--34098fce-860f-48ae-8e50-ebd3cc5e41da",
                    "marking-definition--5e57c739-391a-4eb3-b6be-7d15ca92d5ed",
                    "marking-definition--94868c89-83c2-464b-929b-a1a8aa3c8487",
                    "marking-definition--bab4a63c-aed9-4cf5-a766-dfca5abac2bb",
                    "marking-definition--55d920b0-5e8b-4f79-9ee9-91f868d9b421",
                    "marking-definition--939a9414-2ddd-4d32-a0cd-375ea402b003",
                    "marking-definition--f88d31f6-486f-44da-b317-01333bde0b82",
                    "marking-definition--72e906ce-ca1b-5d73-adcd-9ea9eb66a1b4",
                },
            )

        with subtests.test("get_sros"):
            helper = ArangoDBHelper(
                helper.collection, request_from_queries(include_embedded_refs="false")
            )
            response_data = helper.get_sros().data
            test_no_duplicate(
                response_data,
                {
                    "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                    "relationship--1162f86e-c825-4b20-a69e-ea8a6d9d3948",
                    "relationship--2c8df0f5-1a2b-4b1b-9f13-21a2767c8f0d",
                },
            )

        with subtests.test("get_sdos"):
            helper = ArangoDBHelper(
                helper.collection, request_from_queries(include_embedded_refs="false")
            )
            response_data = helper.get_sdos().data
            test_no_duplicate(
                response_data,
                {
                    "weakness--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                    "indicator--d28eedd5-8066-4b4a-a6e8-a31795403f97",
                    "vulnerability--1162f86e-c825-4b20-a69e-ea8a6d9d3948",
                    ## default identity
                    "identity--72e906ce-ca1b-5d73-adcd-9ea9eb66a1b4",
                    "identity--9779a2db-f98c-5f4b-8d08-8ee04e02dbb5",
                },
            )


sco_objects = [
    {"type": "artifact", "payload_bin": "some unrelated content", "id": "artifact-1"},
    {"type": "autonomous-system", "number": "AS12345", "id": "as-1"},
    {
        "type": "bank-account",
        "iban_number": "DE89370400440532013000",
        "id": "bank-acc-1",
    },
    {"type": "bank-card", "number": "4111111111111111", "id": "bank-card-1"},
    {
        "type": "cryptocurrency-transaction",
        "hash": "abcde98765fghij",
        "id": "crypto-tx-1",
    },
    {"type": "cryptocurrency-wallet", "hash": "wallet-hash-xyz789", "id": "wallet-1"},
    {"type": "directory", "path": "/usr/local/bin", "id": "dir-1"},
    {"type": "domain-name", "value": "example.net", "id": "domain-1"},
    {"type": "email-addr", "value": "user@example.com", "id": "email-1"},
    {"type": "email-message", "body": "Nothing relevant here", "id": "email-msg-1"},
    {"type": "file", "name": "report.pdf", "id": "file-1"},
    {"type": "ipv4-addr", "value": "192.168.0.123", "id": "ipv4-1"},
    {"type": "ipv6-addr", "value": "fe80::5678:abcd", "id": "ipv6-1"},
    {"type": "mac-addr", "value": "aa:bb:cc:dd:ee:ff", "id": "mac-1"},
    {"type": "mutex", "value": "Global\\SomeMutex", "id": "mutex-1"},
    {"type": "network-traffic", "protocols": "tcp,udp,icmp", "id": "net-1"},
    {"type": "phone-number", "number": "+1-123-456-7890", "id": "phone-1"},
    {"type": "process", "pid": "12399", "id": "process-1"},
    {"type": "software", "name": "Antivirus", "id": "software-1"},
    {"type": "url", "value": "https://example.com", "id": "url-1"},
    {"type": "user-account", "display_name": "Regular User", "id": "user-1"},
    {"type": "user-agent", "string": "Mozilla/5.0", "id": "agent-1"},
    {
        "type": "windows-registry-key",
        "key": "HKEY_LOCAL_MACHINE\\Software\\Key\\Subkey123",
        "id": "regkey-1",
    },
    {"type": "x509-certificate", "subject": "CN=TrustedCert Authority", "id": "cert-1"},
]

SCO_VALUE_FILTER_TEST_DATA = test_data = [
    pytest.param(
        [
            {
                "type": "artifact",
                "payload_bin": "some static content",
                "id": "artifact-1",
            }
        ],
        "static",
        ["artifact-1"],
        id="match-artifact-on-payload_bin",
    ),
    pytest.param(
        [{"type": "autonomous-system", "number": "AS12345", "id": "as-1"}],
        "123",
        ["as-1"],
        id="match-autonomous-system-on-number",
    ),
    pytest.param(
        [
            {
                "type": "bank-account",
                "iban_number": "DE89370400440532013000",
                "id": "bank-acc-1",
            }
        ],
        "3201",
        ["bank-acc-1"],
        id="match-bank-account-on-iban_number",
    ),
    pytest.param(
        [{"type": "bank-card", "number": "4111111111111111", "id": "bank-card-1"}],
        "1111",
        ["bank-card-1"],
        id="match-bank-card-on-number",
    ),
    pytest.param(
        [
            {
                "type": "cryptocurrency-transaction",
                "hash": "abcde12345fghij",
                "id": "crypto-tx-1",
            }
        ],
        "12345",
        ["crypto-tx-1"],
        id="match-crypto-tx-on-hash",
    ),
    pytest.param(
        [
            {
                "type": "cryptocurrency-wallet",
                "hash": "wallet-hash-xyz789",
                "id": "wallet-1",
            }
        ],
        "xyz",
        ["wallet-1"],
        id="match-crypto-wallet-on-hash",
    ),
    pytest.param(
        [{"type": "directory", "path": "/usr/local/static/bin", "id": "dir-1"}],
        "static",
        ["dir-1"],
        id="match-directory-on-path",
    ),
    pytest.param(
        [{"type": "domain-name", "value": "malicious-static.net", "id": "domain-1"}],
        "static",
        ["domain-1"],
        id="match-domain-name-on-value",
    ),
    pytest.param(
        [{"type": "email-addr", "value": "attacker@static.com", "id": "email-1"}],
        "static",
        ["email-1"],
        id="match-email-addr-on-value",
    ),
    pytest.param(
        [
            {
                "type": "email-message",
                "body": "This message contains static content.",
                "id": "email-msg-1",
            }
        ],
        "static",
        ["email-msg-1"],
        id="match-email-message-on-body",
    ),
    pytest.param(
        [{"type": "file", "name": "static.exe", "id": "file-1"}],
        "static",
        ["file-1"],
        id="match-file-on-name",
    ),
    pytest.param(
        [{"type": "ipv4-addr", "value": "192.168.1.123", "id": "ipv4-1"}],
        "123",
        ["ipv4-1"],
        id="match-ipv4-addr-on-value",
    ),
    pytest.param(
        [{"type": "ipv6-addr", "value": "fe80::1234:abcd", "id": "ipv6-1"}],
        "1234",
        ["ipv6-1"],
        id="match-ipv6-addr-on-value",
    ),
    pytest.param(
        [{"type": "mac-addr", "value": "aa:bb:cc:dd:ee:ff", "id": "mac-1"}],
        "dd:ee",
        ["mac-1"],
        id="match-mac-addr-on-value",
    ),
    pytest.param(
        [{"type": "mutex", "value": "Global\\StaticMutexName", "id": "mutex-1"}],
        "static",
        ["mutex-1"],
        id="match-mutex-on-value",
    ),
    pytest.param(
        [{"type": "network-traffic", "protocols": "tcp,udp,icmp", "id": "net-1"}],
        "icmp",
        ["net-1"],
        id="match-network-traffic-on-protocols",
    ),
    pytest.param(
        [{"type": "phone-number", "number": "+1-234-567-8901", "id": "phone-1"}],
        "234",
        ["phone-1"],
        id="match-phone-number-on-number",
    ),
    pytest.param(
        [{"type": "process", "pid": "12399", "id": "process-1"}],
        "123",
        ["process-1"],
        id="match-process-on-pid",
    ),
    pytest.param(
        [{"type": "software", "name": "StaticScan Antivirus", "id": "software-1"}],
        "static",
        ["software-1"],
        id="match-software-on-name",
    ),
    pytest.param(
        [{"type": "url", "value": "https://staticdomain.com/path", "id": "url-1"}],
        "static",
        ["url-1"],
        id="match-url-on-value",
    ),
    pytest.param(
        [{"type": "user-account", "display_name": "Static User", "id": "user-1"}],
        "static",
        ["user-1"],
        id="match-user-account-on-display_name",
    ),
    pytest.param(
        [{"type": "user-agent", "string": "StaticBot/1.0", "id": "agent-1"}],
        "static",
        ["agent-1"],
        id="match-user-agent-on-string",
    ),
    pytest.param(
        [
            {
                "type": "windows-registry-key",
                "key": "HKEY_LOCAL_MACHINE\\Software\\StaticKey",
                "id": "regkey-1",
            }
        ],
        "statickey",
        ["regkey-1"],
        id="match-registry-key-on-key",
    ),
    pytest.param(
        [
            {
                "type": "x509-certificate",
                "subject": "CN=StaticCert Authority",
                "id": "cert-1",
            }
        ],
        "static",
        ["cert-1"],
        id="match-x509-certificate-on-subject",
    ),
    pytest.param(
        sco_objects,
        "123",
        ["as-1", "regkey-1", "ipv4-1", "phone-1", "process-1"],
        id="match-mixed-all-types-only-5-match-123",
    ),
]


@pytest.mark.parametrize(
    ["objects", "value", "expected_ids"], SCO_VALUE_FILTER_TEST_DATA
)
def test_get_scos_value_filter(objects, value, expected_ids):
    helper = ArangoDBHelper(
        conf.ARANGODB_DATABASE_VIEW, request_from_queries(value=value)
    )
    with make_s2a_uploads(
        [("test_sco_values", objects)], truncate_collection=True
    ) as s2a:
        response_data = helper.get_scos().data
        objects = response_data["objects"]
        object_ids = {obj["id"] for obj in objects}
        assert object_ids == set(expected_ids)


def test_get_scos_type_filter(subtests):
    types = {obj["type"] for obj in sco_objects}
    with make_s2a_uploads(
        [("test_sco_values", sco_objects)], truncate_collection=True
    ) as s2a:
        for _ in range(20):
            expected_types = random.choices(list(types), k=random.randint(5, 10))
            with subtests.test(types=expected_types):
                helper = ArangoDBHelper(
                    conf.ARANGODB_DATABASE_VIEW,
                    request_from_queries(types=",".join(expected_types)),
                )
                response_data = helper.get_scos().data
                objects = response_data["objects"]
                object_types = {obj["type"] for obj in objects}
                assert object_types == set(expected_types)


@pytest.fixture(scope="module")
def sdo_data():
    objects = [
        {
            "type": "weakness",
            "id": "weakness--ac6f22ba-3909-43fa-8f81-1997590a1d7e",
            "labels": ["weak label 1", "strong label 2"],
            "name": "a weakness object",
        },
        {
            "type": "weakness",
            "id": "weakness--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
            "labels": ["weak label 3"],
            "name": "another weakness object",
        },
        {
            "type": "vulnerability",
            "id": "vulnerability--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
            "labels": ["weak label 3"],
            "name": "another vulnerability object",
        },
    ]
    with make_s2a_uploads(
        [("test_sdo_filters", objects)], truncate_collection=True
    ) as s2a:
        yield objects


@pytest.mark.parametrize(
    ["filters", "expected_ids"],
    [
        (
            dict(labels="weak"),
            [
                "vulnerability--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                "weakness--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                "weakness--ac6f22ba-3909-43fa-8f81-1997590a1d7e",
            ],
        ),
        (
            dict(labels="weak", types="weakness"),
            [
                "weakness--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
                "weakness--ac6f22ba-3909-43fa-8f81-1997590a1d7e",
            ],
        ),
        (
            dict(types="weakness", name="another"),
            ["weakness--cbd67181-b9f8-595b-8bc3-3971e34fa1cc"],
        ),
        (dict(labels="strong"), ["weakness--ac6f22ba-3909-43fa-8f81-1997590a1d7e"]),
    ],
)
def test_sdo_filters(sdo_data, filters, expected_ids):
    helper = ArangoDBHelper(
        conf.ARANGODB_DATABASE_VIEW,
        request_from_queries(**filters),
    )
    assert {obj["id"] for obj in helper.get_sdos().data["objects"]} == set(expected_ids)


@pytest.mark.parametrize(
    "stix_id",
    [
        "vulnerability--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
        "weakness--cbd67181-b9f8-595b-8bc3-3971e34fa1cc",
        "weakness--ac6f22ba-3909-43fa-8f81-1997590a1d7e",
        "relationship--8a5a7ecf-56cc-4ca5-947f-088870f54ea9",
        "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
        "relationship--ce65bbc0-5715-4d44-a24f-42b9757d36f4",
    ],
)
def test_get_objects_by_id(sro_data, sdo_data, stix_id):
    helper = ArangoDBHelper(
        conf.ARANGODB_DATABASE_VIEW,
        request_from_queries(),
    )
    data = helper.get_objects_by_id(stix_id).data
    assert data["id"] == stix_id


@pytest.mark.parametrize(
    "stix_id",
    [
        "bad_id1",
        "bad_id2",
        "bad_id3",
    ],
)
def test_get_objects_by_id__bad_id(sro_data, sdo_data, stix_id):
    helper = ArangoDBHelper(
        conf.ARANGODB_DATABASE_VIEW,
        request_from_queries(),
    )
    with pytest.raises(NotFound):
        data = helper.get_objects_by_id(stix_id).data


@pytest.fixture(scope="module")
def sro_data():
    objects = [
        {
            "type": "relationship",
            "id": "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
            "source_ref": "ex-type1--2",
            "target_ref": "ex-type2--2",
            "relationship_type": "created-for",
        },  # SRO1
        {
            "type": "relationship",
            "id": "relationship--1162f86e-c825-4b20-a69e-ea8a6d9d3948",
            "source_ref": "ex-type2--3",
            "target_ref": "ex-type3--3",
            "relationship_type": "killed-by",
        },  # SRO2
        {
            "type": "relationship",
            "id": "relationship--ce65bbc0-5715-4d44-a24f-42b9757d36f4",
            "source_ref": "ex-type3--3",
            "target_ref": "ex-type2--3",
            "relationship_type": "exists-for",
        },  # SRO3
        {
            "type": "relationship",
            "id": "relationship--8a5a7ecf-56cc-4ca5-947f-088870f54ea9",
            "source_ref": "ex-type2--3",
            "target_ref": "ex-type1--3",
            "relationship_type": "exists-for",
            "_is_ref": True,
        },  # SRO3
    ]
    with make_s2a_uploads(
        [("test_sro_filters", objects)], truncate_collection=True
    ) as s2a:
        yield objects


@pytest.mark.parametrize(
    ["filters", "expected_ids"],
    [
        (
            dict(source_ref_type="ex-type2"),
            [
                "relationship--8a5a7ecf-56cc-4ca5-947f-088870f54ea9",
                "relationship--1162f86e-c825-4b20-a69e-ea8a6d9d3948",
            ],
        ),
        (
            dict(target_ref_type="ex-type2"),
            [
                "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                "relationship--ce65bbc0-5715-4d44-a24f-42b9757d36f4",
            ],
        ),
        (
            dict(target_ref_type="ex-type2", source_ref_type="ex-type3"),
            [
                "relationship--ce65bbc0-5715-4d44-a24f-42b9757d36f4",
            ],
        ),
        (
            dict(relationship_type="for"),
            [
                "relationship--8a5a7ecf-56cc-4ca5-947f-088870f54ea9",
                "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                "relationship--ce65bbc0-5715-4d44-a24f-42b9757d36f4",
            ],
        ),
        (
            dict(relationship_type="for", target_ref_type="ex-type2"),
            [
                "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                "relationship--ce65bbc0-5715-4d44-a24f-42b9757d36f4",
            ],
        ),
        (
            dict(include_embedded_refs="false"),
            [
                "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                "relationship--1162f86e-c825-4b20-a69e-ea8a6d9d3948",
                "relationship--ce65bbc0-5715-4d44-a24f-42b9757d36f4",
            ],
        ),
        (
            dict(target_ref="sd"),
            [],
        ),
        (
            dict(target_ref="ex-type1--3"),
            ["relationship--8a5a7ecf-56cc-4ca5-947f-088870f54ea9"],
        ),
        (
            dict(source_ref="ex-type1--2"),
            [
                "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
            ],
        ),
    ],
)
def test_sro_filters(sro_data, filters, expected_ids):
    helper = ArangoDBHelper(
        conf.ARANGODB_DATABASE_VIEW,
        request_from_queries(**filters),
    )
    assert {obj["id"] for obj in helper.get_sros().data["objects"]} == set(expected_ids)


@pytest.fixture(scope="module")
def sro_data():
    objects = SRO_DATA
    with make_s2a_uploads(
        [("test_sro_filters", objects)], truncate_collection=True
    ) as s2a:
        yield objects


green_ref, red_ref, clear_ref = (
    "marking-definition--bab4a63c-aed9-4cf5-a766-dfca5abac2bb",
    "marking-definition--e828b379-4e03-4974-9ac4-e53a884c97c1",
    "marking-definition--94868c89-83c2-464b-929b-a1a8aa3c8487",
)


@pytest.fixture(scope="module")
def bundle_data():
    objects = SRO_DATA + [
        {"id": "ex-type1--2", "type": "ex-type1"},
        {
            "id": "ex-type2--2",
            "type": "ex-type2",
            "created_by_ref": "ref1",
            "object_marking_refs": [green_ref],
        },
        {
            "id": "ex-type2--3",
            "type": "ex-type2",
            "created_by_ref": "ref2",
            "object_marking_refs": [red_ref],
        },
        {"id": "ex-type3--3", "type": "ex-type3"},
        {
            "id": "ex-type1--3",
            "type": "ex-type1",
            "created_by_ref": "ref2",
            "object_marking_refs": [clear_ref],
        },
        {
            "type": "relationship",
            "id": "relationship--red",
            "source_ref": "ex-type1--2",
            "target_ref": "ex-type2--3",
            "created_by_ref": "ref1",
            "object_marking_refs": [red_ref],
            "relationship_type": "exists-for",
        },  # SRO3
        {
            "type": "relationship",
            "id": "relationship--clear",
            "_is_ref": True,
            "source_ref": "ex-type1--2",
            "target_ref": "ex-type1--3",
            "created_by_ref": "ref2",
            "relationship_type": "exists-for",
            "object_marking_refs": [green_ref],
        },  # SRO3
    ]

    with make_s2a_uploads(
        [("test_bundle_filters", objects)], truncate_collection=True
    ) as s2a:
        yield objects


@pytest.mark.parametrize(
    ["stix_id", "filters", "expected_ids"],
    [
        pytest.param(
            "ex-type1--2",
            None,
            [
                "ex-type1--2",
                "ex-type2--2",
                "ex-type2--3",
                "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                "relationship--red",
                "ex-type1--3",
                "relationship--clear",
            ],
            id="no filters",
        ),
        pytest.param(
            "ex-type1--2",
            dict(visible_to="ref1"),
            [
                "ex-type1--2",
                "ex-type2--2",
                "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                "relationship--red",
                "ex-type1--3",
                "relationship--clear",
            ],
            id="visible_to:ref1",
        ),
        pytest.param(
            "ex-type1--2",
            dict(visible_to="ref2"),
            [
                "ex-type1--2",
                "ex-type2--3",
                "ex-type2--2",
                "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                "ex-type1--3",
                "relationship--clear",
            ],
            id="visible_to:ref2",
        ),
        pytest.param(
            "ex-type1--2",
            dict(visible_to="ref2", include_embedded_refs="false"),
            [
                "ex-type1--2",
                "ex-type2--3",
                "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                "ex-type2--2",
            ],
            id="visible_to:ref2, include_embedded_refs:False",
        ),
        pytest.param(
            "ex-type1--2",
            dict(types="ex-type2,relationship", include_embedded_refs="false"),
            [
                "relationship--red",
                "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                "ex-type2--3",
                "ex-type2--2",
            ],
            id="types:list[1], include_embedded_refs:False",
        ),
        pytest.param(
            "ex-type1--2",
            dict(
                types="ex-type2,relationship",
                include_embedded_refs="false",
                visible_to="ref1",
            ),
            [
                "relationship--red",
                "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
                "ex-type2--2",
            ],
            id="types:list[1], include_embedded_refs:False, visible_to:ref1",
        ),
        pytest.param(
            "ex-type1--2",
            dict(types="ex-type2", include_embedded_refs="false", visible_to="ref1"),
            [
                "ex-type2--2",
            ],
            id="types:list[2], include_embedded_refs:False, visible_to:ref1",
        ),
    ],
)
def test_get_object_bundle(bundle_data, stix_id, filters, expected_ids):
    filters = filters or {}
    helper = ArangoDBHelper(
        conf.ARANGODB_DATABASE_VIEW,
        request_from_queries(**filters),
    )
    objects = helper.get_object_bundle(stix_id).data["objects"]
    assert {obj["id"] for obj in objects} == set(expected_ids)

    if visible_to := filters.get("visible_to"):
        for obj in objects:
            assert obj.get("created_by_ref") in [None, visible_to] or not set(
                obj.get("object_marking_refs", [])
            ).isdisjoint([green_ref, clear_ref])

    if types := filters.get("types"):
        types = types.split(",")
        assert {obj["type"] for obj in objects} == set(types)


@pytest.mark.parametrize("path", ["/objects/sdos/", "/objects/sros/"])
@pytest.mark.parametrize(
    "identity_ref",
    [
        "identity--c78cb6e5-0c4b-4611-8297-d1b8b55e40b5",
        "identity--72e906ce-ca1b-5d73-adcd-9ea9eb66a1b4",
        "identity--bad-identity",
    ],
)
def test_visible_to(client, path, identity_ref):
    resp = client.get(path, query_params=dict(visible_to=identity_ref))
    objects = resp.data["objects"]
    for obj in objects:
        assert obj.get("created_by_ref") in [None, identity_ref] or not set(
            obj.get("object_marking_refs", [])
        ).isdisjoint([green_ref, clear_ref])



import random
import pytest
from unittest.mock import MagicMock, patch
from dogesec_commons.objects.helpers import positive_int, ArangoDBHelper


@pytest.mark.parametrize(
    "value,cutoff,default,expected",
    [
        ("10", None, 1, 10),
        ("-5", None, 1, 1),
        ("abc", None, 1, 1),
        ("50", 30, 1, 30),
        (None, None, 2, 2),
    ],
)
def test_positive_int(value, cutoff, default, expected):
    assert positive_int(value, cutoff, default) == expected


@patch("dogesec_commons.objects.helpers.ArangoDBHelper.client")
@pytest.mark.parametrize(
    "params",
    [
        dict(page=2, page_size=5),
        dict(page=5, page_size=51),
        dict(page=5, page_size=51),
        dict(page_size=51),
    ],
)
def test_get_page_params(mock_client, params):
    page, page_size = ArangoDBHelper.get_page_params(params)
    assert page == max(params.get("page", -9), 1)
    assert page_size == min(params.get("page_size", 50), 50)


@pytest.mark.parametrize(
    ["value", "expected"],
    [
        ("malware,file", ["malware", "file"]),
        ("malware", ["malware"]),
        ("", []),
    ],
)
def test_query_as_array(value, expected):
    request = MagicMock()
    request.query_params.dict.return_value = {"types": value}
    helper = ArangoDBHelper("collection", request)
    result = helper.query_as_array("types")
    assert result == expected


@pytest.mark.parametrize(
    ["value", "expected"],
    [
        ("true", True),
        ("True", True),
        ("1", True),
        ("y", True),
        ("0", False),
        ("false", False),
        ("False", False),
        ("no", False),
    ],
)
def test_query_as_bool(value, expected):
    request = MagicMock()
    request.query_params.dict.return_value = {"include_embedded_refs": value}
    helper = ArangoDBHelper("collection", request)
    result = helper.query_as_bool("include_embedded_refs", default=None)
    assert result is expected


def test_query_as_bool_default():
    request = MagicMock()
    request.query_params.dict.return_value = {"exists": "false"}
    helper = ArangoDBHelper("collection", request)
    assert helper.query_as_bool("exists", default=True) == False
    assert helper.query_as_bool("badkey", default=False) == False
    assert helper.query_as_bool("badkey", default=True) == True


@pytest.mark.parametrize(
    "query_sort,sort_options,customs,doc_name,expected",
    [
        # Case 1: Valid sort field, ascending
        (
            "name_ascending",
            ["name_ascending", "date_descending"],
            {},
            "doc",
            "SORT doc.name ASC",
        ),
        # Case 2: Valid sort field, descending
        (
            "date_descending",
            ["name_ascending", "date_descending"],
            {},
            "doc",
            "SORT doc.date DESC",
        ),
        # Case 3: Sort not in options, should return empty string
        ("invalid_sort", ["name_ascending", "date_descending"], {}, "doc", ""),
        # Case 4: Custom field mapping used
        (
            "score_descending",
            ["score_descending"],
            {"score": "custom.score_field"},
            "doc",
            "SORT custom.score_field DESC",
        ),
        # Case 5: Different doc_name prefix
        ("rank_ascending", ["rank_ascending"], {}, "item", "SORT item.rank ASC"),
    ],
)
def test_get_sort_stmt(query_sort, sort_options, customs, doc_name, expected):
    request = MagicMock()
    request.query_params.dict.return_value = {"sort": query_sort}
    helper = ArangoDBHelper("collection", request)
    helper.query = {"sort": query_sort}
    result = helper.get_sort_stmt(sort_options, customs=customs, doc_name=doc_name)
    assert result == expected


def test_get_offset_and_count():
    helper = ArangoDBHelper("collection", MagicMock())
    offset, count = helper.get_offset_and_count(10, 3)
    assert offset == 20
    assert count == 10


def test_get_paginated_response():
    data = [{"name": "Item1"} for _ in range(random.randint(5, 10))]
    response = ArangoDBHelper.get_paginated_response(
        data, page_number=2, page_size=3, full_count=10
    )
    assert response.data["page_size"] == 3
    assert response.data["page_results_count"] == len(data)
    assert response.data["page_number"] == 2
    assert response.data["total_results_count"] == 10
    assert response.data["objects"] == data



# Generated by Django 5.2.4 on 2025-07-09 09:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0006_alter_profile_ai_create_attack_flow'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='profile',
            name='ai_summary_provider',
        ),
        migrations.AddField(
            model_name='profile',
            name='ai_extract_if_no_incidence',
            field=models.BooleanField(default=True),
        ),
    ]



import uuid
import pytest
from django.core.exceptions import ValidationError
from dogesec_commons.stixifier import models


@pytest.mark.django_db
class TestProfileModel:

    def test_profile_creation_generates_uuid(self):
        profile = models.Profile.objects.create(
            name="Test Profile",
            extractions=["some.extractor"],
            defang=True,
            ai_settings_relationships="default",
            ai_settings_extractions=["ex1"],
            ai_content_check_provider="provider",
        )
        assert isinstance(profile.id, uuid.UUID)
        assert profile.name == "Test Profile"
        assert profile.defang is True

    @pytest.mark.parametrize("identity_id", ["identity--xyz", None])
    def test_profile_id_deterministic_with_identity_id(self, identity_id):
        profile1 = models.Profile(
            name="profile-name",
            identity_id=identity_id,
            extractions=["a"],
            defang=True,
            ai_settings_relationships="r",
            ai_settings_extractions=[],
            ai_content_check_provider="p",
        )
        profile1.save()
        profile_1_id = profile1.id
        profile1.delete()
        profile2 = models.Profile(
            name="profile-name",
            identity_id=identity_id,
            extractions=["a"],
            defang=True,
            ai_settings_relationships="r",
            ai_settings_extractions=[],
            ai_content_check_provider="p",
        )
        profile2.save()
        assert profile_1_id == profile2.id

    def test_relationship_mode_choices(self):
        assert models.RelationshipMode.AI == "ai"
        assert models.RelationshipMode.STANDARD == "standard"


def test_validate_extractor_valid():
    assert models.validate_extractor(["pattern"], "pattern_host_name") is True


def test_validate_extractor_invalid():
    with pytest.raises(ValidationError):
        models.validate_extractor(["pattern"], "missing_extractor")



# Generated by Django 5.1.5 on 2025-02-20 11:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0002_profile_ai_content_check_variable'),
    ]

    operations = [
        migrations.RenameField(
            model_name='profile',
            old_name='ai_content_check_variable',
            new_name='ai_content_check_provider',
        ),
        migrations.AddField(
            model_name='profile',
            name='ai_create_attack_flow',
            field=models.BooleanField(default=True),
        ),
    ]



"""
URL configuration for dogesec_commons project.

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/5.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.contrib import admin
from django.urls import include, path
from rest_framework import routers
from dogesec_commons.stixifier.views import ExtractorsView, ProfileView
from drf_spectacular.views import SpectacularAPIView, SpectacularSwaggerView
from dogesec_commons.objects import views as arango_views
router = routers.SimpleRouter(use_regex_path=False)

router.register('profiles', ProfileView, "profile-view")
# txt2stix views
router.register('extractors', ExtractorsView, "extractors-view")

## objects
regex_router = routers.SimpleRouter(use_regex_path=True)
regex_router.register("", arango_views.ObjectsWithReportsView, "object-view-orig")
regex_router.register('smos', arango_views.SMOView, "object-view-smo")
regex_router.register('scos', arango_views.SCOView, "object-view-sco")
regex_router.register('sros', arango_views.SROView, "object-view-sro")
regex_router.register('sdos', arango_views.SDOView, "object-view-sdo")
urlpatterns = [
    path(f'api/', include(router.urls)),
    path(f'objects/', include(regex_router.urls)),
    path('admin/', admin.site.urls),

    # YOUR PATTERNS
    path('api/schema/', SpectacularAPIView.as_view(), name='schema'),
    # Optional UI:
    path('api/schema/swagger-ui/', SpectacularSwaggerView.as_view(url_name='schema'), name='swagger-ui'),
]



SRO_DATA = [
    {
        "type": "relationship",
        "id": "relationship--9cf0369a-8646-4979-ae2c-ab0d3c95bfad",
        "source_ref": "ex-type1--2",
        "target_ref": "ex-type2--2",
        "relationship_type": "created-for",
    },  # SRO1
    {
        "type": "relationship",
        "id": "relationship--1162f86e-c825-4b20-a69e-ea8a6d9d3948",
        "source_ref": "ex-type2--3",
        "target_ref": "ex-type3--3",
        "relationship_type": "killed-by",
    },  # SRO2
    {
        "type": "relationship",
        "id": "relationship--ce65bbc0-5715-4d44-a24f-42b9757d36f4",
        "source_ref": "ex-type3--3",
        "target_ref": "ex-type2--3",
        "relationship_type": "exists-for",
    },  # SRO3
    {
        "type": "relationship",
        "id": "relationship--8a5a7ecf-56cc-4ca5-947f-088870f54ea9",
        "source_ref": "ex-type2--3",
        "target_ref": "ex-type1--3",
        "relationship_type": "exists-for",
        "_is_ref": True,
    },  # SRO3
]


"""
Django settings for dogesec_commons project.

Generated by 'django-admin startproject' using Django 5.1.5.

For more information on this file, see
https://docs.djangoproject.com/en/5.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/5.1/ref/settings/
"""

import os
from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/5.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'django-insecure-umqh5d5yygalt#x%ekanq_liz6c17fts)w!p=i4d$2ll%t2kro'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'dogesec_commons.stixifier',
    'dogesec_commons.objects',
    'drf_spectacular',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'dogesec_commons.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'dogesec_commons.wsgi.application'


# Database
# https://docs.djangoproject.com/en/5.1/ref/settings/#databases


DATABASES = {
    
    'default': {
        'ENGINE': 'django.db.backends.postgresql',
        'NAME': os.getenv('POSTGRES_DB'),            # Database name
        'USER': os.getenv('POSTGRES_USER'),          # Database user
        'PASSWORD': os.getenv('POSTGRES_PASSWORD'),  # Database password
        'HOST': os.getenv('POSTGRES_HOST'),          # PostgreSQL service name in Docker Compose
        'PORT': os.getenv('POSTGRES_PORT', '5432'),  # PostgreSQL default port
    },
    
}


# Password validation
# https://docs.djangoproject.com/en/5.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/5.1/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/5.1/howto/static-files/

STATIC_URL = 'static/'

# Default primary key field type
# https://docs.djangoproject.com/en/5.1/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

MAXIMUM_PAGE_SIZE = int(os.getenv("MAX_PAGE_SIZE", 50))
DEFAULT_PAGE_SIZE = int(os.getenv("DEFAULT_PAGE_SIZE", 50))


SPECTACULAR_SETTINGS = {
    # available SwaggerUI configuration parameters
    # https://swagger.io/docs/open-source-tools/swagger-ui/usage/configuration/
    "SWAGGER_UI_SETTINGS": {
        "deepLinking": True,
        "persistAuthorization": True,
        "displayOperationId": True,
    },
    # available SwaggerUI versions: https://github.com/swagger-api/swagger-ui/releases
    "SWAGGER_UI_DIST": "https://cdn.jsdelivr.net/npm/swagger-ui-dist@latest", # default
}

REST_FRAMEWORK = {
    # YOUR SETTINGS
    'DEFAULT_SCHEMA_CLASS': 'drf_spectacular.openapi.AutoSchema',
    'EXCEPTION_HANDLER': "dogesec_commons.utils.custom_exception_handler",
}

ARANGODB_DATABASE = os.getenv('ARANGODB_DATABASE')
ARANGODB_USERNAME = os.getenv('ARANGODB_USERNAME')
ARANGODB_PASSWORD = os.getenv('ARANGODB_PASSWORD')
ARANGODB_HOST_URL = os.getenv("ARANGODB_HOST_URL")


import uuid
import pytest
from pytest_django.fixtures import skip_if_no_django, SettingsWrapper




@pytest.fixture(autouse=True, scope="package")
def s_settings():
    skip_if_no_django()

    settings = SettingsWrapper()
    settings.STIXIFIER_NAMESPACE = uuid.uuid4()
    settings.GOOGLE_VISION_API_KEY = settings.ARANGODB_DATABASE_VIEW = settings.INPUT_TOKEN_LIMIT = None
    yield settings
    settings.finalize()


import uuid
import pytest
from dogesec_commons.stixifier.serializers import validate_model, validate_stix_id
from rest_framework.validators import ValidationError


def test_validate_model():
    assert validate_model("openai") == "openai"
    assert validate_model("") == None
    with pytest.raises(ValidationError):
        validate_model("random:10s")


def test_validate_stix_id():
    id = str(uuid.uuid4())
    assert validate_stix_id("report--" + id, "report") == "report--" + id
    assert validate_stix_id("indicator--" + id, "indicator") == "indicator--" + id
    with pytest.raises(ValidationError):
        validate_stix_id("indicator--" + id, "report")
    with pytest.raises(ValidationError):
        validate_stix_id("indicator--bad-id", "indicator")



from unittest.mock import MagicMock
from datetime import datetime
from django.utils.timezone import make_aware
from dogesec_commons.utils.filters import MinMaxDateFilter


def test_filter_queryset_with_minmax_dates():
    filter_backend = MinMaxDateFilter()
    mock_view = MagicMock()
    mock_view.minmax_date_fields = ["created", "modified"]

    mock_request = MagicMock()
    mock_request.query_params = {
        "created_min": "2024-01-01T00:00:00Z",
        "created_max": "2024-01-05T00:00:00Z",
    }

    mock_qs = MagicMock()
    filtered_qs = MagicMock()
    mock_qs.filter.return_value = filtered_qs

    expected_gte = make_aware(datetime(2024, 1, 1, 0, 0))
    expected_lte = make_aware(datetime(2024, 1, 5, 0, 0))

    result = filter_backend.filter_queryset(mock_request, mock_qs, mock_view)

    assert result == filtered_qs

    mock_qs.filter.assert_called_once_with(
        created__gte=expected_gte,
        created__lte=expected_lte,
    )

    mock_request.query_params = {
        "created_max": "2024-01-05T00:00:00Z",
    }
    mock_qs.filter.reset_mock()
    result = filter_backend.filter_queryset(mock_request, mock_qs, mock_view)
    mock_qs.filter.assert_called_once_with(
        created__lte=expected_lte,
    )

    mock_request.query_params = {
        "created_min": "2024-01-01T00:00:00Z",
        "modified_max": "2024-01-05T00:00:00Z",
    }
    mock_qs.filter.reset_mock()
    result = filter_backend.filter_queryset(mock_request, mock_qs, mock_view)
    mock_qs.filter.assert_called_once_with(
        created__gte=expected_gte,
        modified__lte=expected_lte,
    )

    mock_request.query_params = {
        "bad_field_min": "2024-01-01T00:00:00Z",
        "modified_max": "2024-01-05T00:00:00Z",
    }
    mock_qs.filter.reset_mock()
    result = filter_backend.filter_queryset(mock_request, mock_qs, mock_view)
    mock_qs.filter.assert_called_once_with(
        modified__lte=expected_lte,
    )



import argparse
import contextlib
from functools import partial
import uuid
from rest_framework import serializers

from . import conf
from .models import Profile
from rest_framework import serializers
import txt2stix.extractions
import txt2stix.txt2stix
from urllib.parse import urljoin
from django.conf import settings
from django.contrib.postgres.fields import ArrayField
from rest_framework.validators import ValidationError
from dogesec_commons.utils.serializers import CommonErrorSerializer

from drf_spectacular.utils import OpenApiResponse, OpenApiExample

from drf_spectacular.utils import OpenApiResponse, OpenApiExample

from django.db import models


def validate_model(model):
    if not model:
        return None
    try:
        extractor = txt2stix.txt2stix.parse_model(model)
    except BaseException as e:
        raise ValidationError(f"invalid model: {model}")
    return model

def validate_extractor(typestr, types, name):
    extractors = txt2stix.extractions.parse_extraction_config(
            txt2stix.txt2stix.INCLUDES_PATH
        )
    if  name not in extractors or extractors[name].type not in types:
        raise ValidationError(f"`{name}` is not a valid {typestr}", 400)
    
def validate_stix_id(stix_id: str, type: str):
    type_part, _, id_part = stix_id.partition('--')
    if type_part != type:
        raise ValidationError(f"Invalid STIX ID for type `{type}`")
    with contextlib.suppress(Exception):
        uuid.UUID(id_part)
        return stix_id
    raise ValidationError("Invalid STIX ID")


def uses_ai(slugs):
    extractors = txt2stix.extractions.parse_extraction_config(
            txt2stix.txt2stix.INCLUDES_PATH
        )
    ai_based_extractors = []
    for slug in slugs:
        if extractors[slug].type == 'ai':
            ai_based_extractors.append(slug)
    
    if ai_based_extractors:
        raise ValidationError(f'AI based extractors `{ai_based_extractors}` used when `ai_settings_extractions` is not configured')

class ProfileSerializer(serializers.ModelSerializer):
    id = serializers.UUIDField(read_only=True)
    identity_id = serializers.CharField(
        max_length=46,
        validators=[lambda stix_id: validate_stix_id(stix_id, "identity")],
        allow_null=True,
        required=False,
        help_text="STIX Identity ID (e.g `identity--19686d47-3a50-48a0-8ef0-f3e0f8a4bd99`)"
    )

    ai_settings_relationships = serializers.CharField(
        validators=[validate_model],
        help_text='(required if AI relationship enabled): passed in format `provider:model`. Can only pass one model at this time.',
        allow_null=True,
        required=False,
    )
    ai_settings_extractions = serializers.ListField(
        child=serializers.CharField(max_length=256, validators=[validate_model]),
        help_text='(required if AI extractions enabled) passed in format provider[:model] e.g. openai:gpt4o. Can pass more than one value to get extractions from multiple providers. model part is optional',
        required=False,
    )
    ai_content_check_provider = serializers.CharField(
        max_length=256, validators=[validate_model],
        allow_null=True,
        required=False,
        help_text='check content before proceeding'
    )
    ai_extract_if_no_incidence = serializers.BooleanField(default=True, help_text="(boolean, default `true`) if content check decides the report is not related to cyber security intelligence (e.g. vendor marketing), then you can use this setting to decide wether or not script should proceed. Setting to `false` will stop processing. It is designed to save AI tokens processing unknown content at scale in an automated way.")
    ai_create_attack_flow = serializers.BooleanField(required=False, help_text="should create attack-flow (default is `false`)", default=False)
    ai_create_attack_navigator_layer = serializers.BooleanField(required=False, help_text="should create attack navigator layer (default is `false`)", default=False)
    extractions = serializers.ListField(
        min_length=1,
        child=serializers.CharField(max_length=256, validators=[partial(validate_extractor, 'extractor', ["ai", "pattern", "lookup"])]),
        help_text="extraction id(s)",
    )
    defang = serializers.BooleanField(help_text='If the text should be defanged before processing')


    ignore_embedded_relationships     = serializers.BooleanField(required=False, help_text="applies to SDO and SCO types (default is `false`)")
    ignore_embedded_relationships_sro = serializers.BooleanField(required=False, help_text="sets wether to ignore embedded refs on `relationship` object types (default is `true`)")
    ignore_embedded_relationships_smo = serializers.BooleanField(required=False, help_text="sets wether to ignore embedded refs on SMO object types (`marking-definition`, `extension-definition`, `language-content`) (default is `true`)")
    generate_pdf = serializers.BooleanField(required=False, help_text="Whether or not to generate pdf file for input, applies to both stixify and obstracts (default is `false`)")

    class Meta:
        model = Profile
        fields = "__all__"

    def validate(self, attrs):
        if not attrs.get('ai_settings_relationships'):
            if attrs['relationship_mode'] == 'ai':
                raise ValidationError('`ai_settings_relationships` is required when `relationship_mode == "ai"`')
            if attrs['ai_create_attack_flow']:
                raise ValidationError('`ai_settings_relationships` is required when `ai_create_attack_flow == true`')
            if attrs['ai_create_attack_navigator_layer']:
                raise ValidationError('`ai_settings_relationships` is required when `ai_create_attack_navigator_layer == true`')
        if not attrs.get('ai_settings_extractions'):
            uses_ai(attrs['extractions'])
        return super().validate(attrs)



DEFAULT_400_ERROR = OpenApiResponse(
    CommonErrorSerializer,
    "The server did not understand the request",
    [
        OpenApiExample(
            "http400",
            {"message": " The server did not understand the request", "code": 400},
        )
    ],
)


DEFAULT_404_ERROR = OpenApiResponse(
    CommonErrorSerializer,
    "Resource not found",
    [
        OpenApiExample(
            "http404",
            {
                "message": "The server cannot find the resource you requested",
                "code": 404,
            },
        )
    ],
)


##



class Txt2stixExtractorSerializer(serializers.Serializer):
    id = serializers.CharField(label='The `id` of the extractor')
    name = serializers.CharField()
    type = serializers.CharField()
    description = serializers.CharField(required=False, allow_null=True)
    notes = serializers.CharField(required=False, allow_null=True)
    file = serializers.CharField(required=False, allow_null=True)
    created = serializers.CharField(required=False, allow_null=True)
    modified = serializers.CharField(required=False, allow_null=True)
    created_by = serializers.CharField(required=False, allow_null=True)
    version = serializers.CharField()
    stix_mapping = serializers.CharField(required=False, allow_null=True)
    dogesec_web = serializers.BooleanField(required=False, allow_null=True)

    @classmethod
    def all_extractors(cls, types):
        retval = {}
        extractors = txt2stix.extractions.parse_extraction_config(
            txt2stix.txt2stix.INCLUDES_PATH
        ).values()
        for extractor in extractors:
            if extractor.type in types:
                retval[extractor.slug] = cls.cleanup_extractor(extractor)
                if extractor.file:
                    retval[extractor.slug]["file"] = urljoin(conf.TXT2STIX_INCLUDE_URL, str(extractor.file.relative_to(txt2stix.txt2stix.INCLUDES_PATH)))
        return retval
    
    @classmethod
    def cleanup_extractor(cls, dct: dict):
        KEYS = cls(data={}).get_fields()
        retval = {"id": dct["slug"]}
        for key in KEYS:
            if key in dct:
                retval[key] = dct[key]
        return retval




# Generated by Django 5.1.5 on 2025-02-18 11:00

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.UUIDField(primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('name', models.CharField(max_length=250, unique=True)),
                ('extractions', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=256), size=None)),
                ('relationship_mode', models.CharField(choices=[('ai', 'AI Relationship'), ('standard', 'Standard Relationship')], default='standard', max_length=20)),
                ('extract_text_from_image', models.BooleanField(default=False)),
                ('defang', models.BooleanField()),
                ('ai_settings_relationships', models.CharField(max_length=256, null=True)),
                ('ai_settings_extractions', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=256), default=list, size=None)),
                ('ai_summary_provider', models.CharField(max_length=256, null=True)),
                ('ignore_image_refs', models.BooleanField(default=True)),
                ('ignore_link_refs', models.BooleanField(default=True)),
                ('ignore_extraction_boundary', models.BooleanField(default=False)),
                ('ignore_embedded_relationships_sro', models.BooleanField(default=True)),
                ('ignore_embedded_relationships_smo', models.BooleanField(default=True)),
                ('ignore_embedded_relationships', models.BooleanField(default=False)),
            ],
        ),
    ]



from dogesec_commons.objects import conf
from dogesec_commons.utils.schemas import DEFAULT_400_RESPONSE
from .helpers import (
    OBJECT_TYPES,
    ArangoDBHelper,
    SCO_TYPES,
    SDO_TYPES,
    SMO_TYPES,
    SRO_SORT_FIELDS,
    SMO_SORT_FIELDS,
    SCO_SORT_FIELDS,
    SDO_SORT_FIELDS,
)
from drf_spectacular.utils import extend_schema_view, extend_schema, OpenApiParameter
from drf_spectacular.types import OpenApiTypes
from rest_framework import viewsets, exceptions, decorators
from rest_framework.response import Response

from django.conf import settings

import textwrap


class QueryParams:
    value = OpenApiParameter(
        "value",
        description=textwrap.dedent(
            """
            Search by the `value` field field of the SCO. This is the IoC. So if you're looking to retrieve a IP address by address you would enter the IP address here. Similarly, if you're looking for a credit card you would enter the card number here.
            Search is wildcard. For example, `1.1` will return SCOs with `value` fields; `1.1.1.1`, `2.1.1.2`, etc.
            If `value` field is named differently for the Object (e.g. `hash`) it will still be searched because these have been aliased to the `value` in the database search).
            """
        ),
    )
    sco_types = OpenApiParameter(
        "types",
        many=True,
        explode=False,
        description=textwrap.dedent(
            """
            Filter the results by one or more STIX SCO Object types
            """
        ),
        enum=SCO_TYPES,
    )
    post_id = OpenApiParameter(
        "post_id",
        description=textwrap.dedent(
            """
            Filter the results to only contain objects present in the specified Post ID. Get a Post ID using the Feeds endpoints.
            """
        ),
    )
    SCO_PARAMS = [
        value,
        sco_types,
        post_id,
        OpenApiParameter("sort", enum=SCO_SORT_FIELDS),
    ]

    name = OpenApiParameter(
        "name",
        description=textwrap.dedent(
            """
            Allows results to be filtered on the `name` field of the SDO. Search is wildcard. For example, `Wanna` will return SDOs with the `name`; `WannaCry`, `WannaSmile`, etc.
            """
        ),
    )
    labels = OpenApiParameter(
        "labels",
        description=textwrap.dedent(
            """
            Allows results to be filtered on each value in the `labels` field of the SDO. Each value in the `labels` list will be searched individually.
            Search is wildcard. For example, `needs` will return SDOs with `labels`; `need-attribution`, `needs-review`, etc. The value entered only needs to match one item in the `labels` list to return results.
            """
        ),
    )
    sdo_types = OpenApiParameter(
        "types",
        many=True,
        explode=False,
        description=textwrap.dedent(
            """
            Filter the results by one or more STIX Domain Object types
            """
        ),
        enum=SDO_TYPES,
    )

    SDO_PARAMS = [
        name,
        labels,
        sdo_types,
        OpenApiParameter("sort", enum=SDO_SORT_FIELDS),
    ]

    source_ref = OpenApiParameter(
        "source_ref",
        description=textwrap.dedent(
            """
            Filter the results on the `source_ref` fields. The value entered should be a full ID of a STIX SDO or SCO which can be obtained from the respective Get Object endpoints. This endpoint allows for graph traversal use-cases as it returns STIX `relationship` objects that will tell you what objects are related to the one entered (in the `target_ref` property).
            """
        ),
    )
    source_ref_type = OpenApiParameter(
        "source_ref_type",
        many=True,
        explode=False,
        description=textwrap.dedent(
            """
            Filter the results by the STIX object type in the `source_ref` field. Unlike the `source_ref` filter that requires a full STIX object ID, this filter allows for a more open search. For example, `attack-pattern` will return all `relationship` Objects where the `source_ref` contains the ID of an `attack-pattern` Object.
            """
        ),
    )
    target_ref = OpenApiParameter(
        "target_ref",
        description=textwrap.dedent(
            """
            Filter the results on the `target_ref` fields. The value entered should be a full ID of a STIX SDO or SCO which can be obtained from the respective Get Object endpoints. This endpoint allows for graph traversal use-cases as it returns STIX `relationship` objects that will tell you what objects are related to the one entered (in the `source_ref` property).
            """
        ),
    )
    target_ref_type = OpenApiParameter(
        "target_ref_type",
        many=True,
        explode=False,
        description=textwrap.dedent(
            """
            Filter the results by the STIX object type in the `target_ref` field. Unlike the `target_ref` filter that requires a full STIX object ID, this filter allows for a more open search. For example, `attack-pattern` will return all `relationship` Objects where the `target_ref` contains the ID of an `attack-pattern` Object.
            """
        ),
    )
    relationship_type = OpenApiParameter(
        "relationship_type",
        description=textwrap.dedent(
            """
            Filter the results on the `relationship_type` field. Search is wildcard. For example, `in` will return `relationship` objects with ``relationship_type`s; `found-in`, `located-in`, etc.
            """
        ),
    )
    include_embedded_refs = OpenApiParameter(
        "include_embedded_refs",
        description=textwrap.dedent(
            """
            If `ignore_embedded_relationships` is set to `false` in the POST request to download data, stix2arango will create SROS for embedded relationships (e.g. from `created_by_refs`). You can choose to show them (`true`) or hide them (`false`) using this parameter. Default value if not passed is `true`.
            """
        ),
        type=OpenApiTypes.BOOL,
    )

    SRO_PARAMS = [
        source_ref,
        source_ref_type,
        target_ref,
        target_ref_type,
        relationship_type,
        include_embedded_refs,
        OpenApiParameter("sort", enum=SRO_SORT_FIELDS),
    ]

    all_types = OpenApiParameter(
        "types",
        many=True,
        explode=False,
        description=textwrap.dedent(
            """
            Filter the results by one or more STIX Object types
            """
        ),
        enum=OBJECT_TYPES,
    )
    OBJECTS_PARAMS = [
        all_types,
    ]

    smo_types = OpenApiParameter(
        "types",
        many=True,
        explode=False,
        description=textwrap.dedent(
            """
            Filter the results by one or more STIX Object types
            """
        ),
        enum=SMO_TYPES,
    )
    SMO_PARAMS = [
        smo_types,
        OpenApiParameter("sort", enum=SMO_SORT_FIELDS),
    ]

    object_id_param = OpenApiParameter(
        "object_id",
        description="Filter by the STIX object ID. e.g. `ipv4-addr--ba6b3f21-d818-4e7c-bfff-765805177512`, `indicator--7bff059e-6963-4b50-b901-4aba20ce1c01`",
        type=OpenApiTypes.STR,
        location=OpenApiParameter.PATH,
    )

    visible_to = OpenApiParameter(
        "visible_to",
        description="Only show objects that are visible to the Identity `id` passed. e.g. passing `identity--b1ae1a15-6f4b-431e-b990-1b9678f35e15` would only show reports created by that identity (with any TLP level) or objects created by another identity ID but only if they are marked with `TLP:CLEAR` or `TLP:GREEN`.",
        type=OpenApiTypes.STR,
    )


@extend_schema_view(
    retrieve=extend_schema(
        summary="Get a STIX Object",
        description=textwrap.dedent(
            """
            Get a STIX Object by its ID
            """
        ),
        responses={200: ArangoDBHelper.STIX_OBJECT_SCHEMA, 404: DEFAULT_400_RESPONSE, 400: DEFAULT_400_RESPONSE},
        parameters=[QueryParams.object_id_param],
    ),
    bundle=extend_schema(
        summary="Get STIX Object's Bundle",
        description=textwrap.dedent(
            """
            Return all objects the STIX Object has a relationship to as a bundle of all objects.
            """
        ),
        responses=ArangoDBHelper.get_paginated_response_schema(),
        parameters=ArangoDBHelper.get_schema_operation_parameters()
        + [
            QueryParams.object_id_param,
            QueryParams.all_types,
            QueryParams.include_embedded_refs,
            QueryParams.visible_to,
        ],
    ),
)
class SingleObjectView(viewsets.ViewSet):
    lookup_url_kwarg = "object_id"
    openapi_tags = ["Objects"]
    lookup_value_regex = (
        r"[\w\-]+--[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}"
    )

    def retrieve(self, request, *args, **kwargs):
        return ArangoDBHelper(conf.ARANGODB_DATABASE_VIEW, request).get_objects_by_id(
            kwargs.get(self.lookup_url_kwarg)
        )

    @decorators.action(detail=True, methods=["GET"])
    def bundle(self, request, *args, **kwargs):
        return ArangoDBHelper(conf.ARANGODB_DATABASE_VIEW, request).get_object_bundle(
            kwargs.get(self.lookup_url_kwarg)
        )


@extend_schema_view(
    reports=extend_schema(
        responses=ArangoDBHelper.get_paginated_response_schema(
            "reports",
            {
                "type": "object",
                "properties": {
                    "type": {
                        "example": "report",
                    },
                    "id": {
                        "example": "report--a86627d4-285b-5358-b332-4e33f3ec1075",
                    },
                },
                "additionalProperties": True,
            },
        ),
        parameters=ArangoDBHelper.get_schema_operation_parameters()
        + [QueryParams.object_id_param],
        summary="Get all Reports that contain this STIX Object",
        description=textwrap.dedent(
            """
            Return all reports the STIX Object has a Relationship to.
            """
        ),
    ),
    destroy_in_report=extend_schema(
        summary="Delete an Object from a Report",
        description=textwrap.dedent(
            """
            Occasionally txt2stix will create an erroneous extraction from a text document. This endpoint allows you to remove the STIX objects created for such extractions.

            This request will delete the object ID specified in the request, and ALL relationship objects that reference this Objects ID in either the `source_ref` or `target_ref` property of the relationship object.

            You can safely to run this request on SCOs that are seen in multiple reports. Whilst Obstracts shows a single SCO with the same value with the same STIX ID (e.g. `1.1.1.1`), in the database multiple versions of the same SCO object exist, one for each report (identified using the field `_stix2arango=<REPORT_ID`). All objects created during the processing of this report have this field too.

            **DANGER:** This action is irreversible. It also can create issues if not run carefully. For example, assume you have a connection from a report object as follows; Report -> Indicator -> IPv4. Should you delete the Indicator objects in this example, the IPv4 will no longer be directly connected to the report (though it will still appear in the report bundle, and the reports `object_refs` property).
            """
        ),
    ),
)
class ObjectsWithReportsView(SingleObjectView):
    @decorators.action(detail=True, methods=["GET"])
    def reports(self, request, *args, **kwargs):
        return ArangoDBHelper(
            conf.ARANGODB_DATABASE_VIEW, request, "reports"
        ).get_containing_reports(kwargs.get(self.lookup_url_kwarg))

    @decorators.action(
        detail=True,
        methods=["DELETE"],
        url_path=r"reports/(?P<report_id>report--[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12})",
    )
    def destroy_in_report(
        self, request, *args, object_id=None, report_id=None, **kwargs
    ):
        return ArangoDBHelper(
            conf.ARANGODB_DATABASE_VIEW, request
        ).delete_report_object(report_id=report_id, object_id=object_id)


@extend_schema_view(
    list=extend_schema(
        responses=ArangoDBHelper.get_paginated_response_schema(),
        parameters=ArangoDBHelper.get_schema_operation_parameters()
        + QueryParams.SDO_PARAMS + [QueryParams.visible_to],
        summary="Search and filter STIX Domain Objects",
        description=textwrap.dedent(
            """
            Search for domain objects (aka TTPs). If you have the object ID already, you can use the base GET Objects endpoint.
            """
        ),
    ),
)
class SDOView(viewsets.ViewSet):
    skip_list_view = True
    openapi_tags = ["Objects"]

    def list(self, request, *args, **kwargs):
        return ArangoDBHelper(conf.ARANGODB_DATABASE_VIEW, request).get_sdos()


@extend_schema_view(
    list=extend_schema(
        responses=ArangoDBHelper.get_paginated_response_schema(),
        parameters=ArangoDBHelper.get_schema_operation_parameters()
        + QueryParams.SCO_PARAMS,
        summary="Search and filter STIX Cyber Observable Objects",
        description=textwrap.dedent(
            """
            Search for STIX Cyber Observable Objects (aka Indicators of Compromise). If you have the object ID already, you can use the base GET Objects endpoint.

            Note the `value` filter searches the following object properties;

            * `artifact.payload_bin`
            * `autonomous-system.number`
            * `bank-account.iban_number`
            * `bank-card.number`
            * `cryptocurrency-transaction.hash`
            * `cryptocurrency-wallet.hash`
            * `directory.path`
            * `domain-name.value`
            * `email-addr.value`
            * `email-message.body`
            * `file.name`
            * `ipv4-addr.value`
            * `ipv6-addr.value`
            * `mac-addr.value`
            * `mutex.value`
            * `network-traffic.protocols`
            * `phone-number.number`
            * `process.pid`
            * `software.name`
            * `url.value`
            * `user-account.display_name`
            * `user-agent.string`
            * `windows-registry-key.key`
            * `x509-certificate.subject`
            """
        ),
    ),
)
class SCOView(viewsets.ViewSet):
    skip_list_view = True
    openapi_tags = ["Objects"]

    def list(self, request, *args, **kwargs):
        matcher = {}
        if post_id := request.query_params.dict().get("post_id"):
            matcher["_obstracts_post_id"] = post_id
        return ArangoDBHelper(conf.ARANGODB_DATABASE_VIEW, request).get_scos(
            matcher=matcher
        )


@extend_schema_view(
    list=extend_schema(
        responses=ArangoDBHelper.get_paginated_response_schema(),
        parameters=ArangoDBHelper.get_schema_operation_parameters()
        + QueryParams.SMO_PARAMS,
        summary="Search and filter STIX Meta Objects",
        description=textwrap.dedent(
            """
            Search for meta objects. If you have the object ID already, you can use the base GET Objects endpoint.
            """
        ),
    )
)
class SMOView(viewsets.ViewSet):
    skip_list_view = True
    openapi_tags = ["Objects"]

    def list(self, request, *args, **kwargs):
        return ArangoDBHelper(conf.ARANGODB_DATABASE_VIEW, request).get_smos()


@extend_schema_view(
    list=extend_schema(
        responses=ArangoDBHelper.get_paginated_response_schema(),
        parameters=ArangoDBHelper.get_schema_operation_parameters()
        + QueryParams.SRO_PARAMS + [QueryParams.visible_to],
        summary="Search and filter STIX Relationship Objects",
        description=textwrap.dedent(
            """
            Search for relationship objects. This endpoint is particularly useful to search what Objects an SCO or SDO is linked to.
            """
        ),
    ),
)
class SROView(viewsets.ViewSet):
    skip_list_view = True
    openapi_tags = ["Objects"]

    def list(self, request, *args, **kwargs):
        return ArangoDBHelper(conf.ARANGODB_DATABASE_VIEW, request).get_sros()



import uuid
from django.urls import include, path
import pytest
from rest_framework.test import APIRequestFactory
from rest_framework.response import Response
from unittest.mock import patch
from rest_framework.test import URLPatternsTestCase
from rest_framework import routers

from dogesec_commons.objects.views import ObjectsWithReportsView, SCOView, SDOView, SMOView, SROView

factory = APIRequestFactory()


@pytest.mark.django_db
@patch("dogesec_commons.objects.views.ArangoDBHelper.get_scos")
def test_sco_view_list_success(mock_get_scos):
    mock_get_scos.return_value = Response({"results": ["filtered-sco"]})
    request = factory.get("/api/objects/sco/")
    response = SCOView.as_view({"get": "list"})(request)
    mock_get_scos.assert_called_once()
    assert response == mock_get_scos.return_value


@pytest.mark.django_db
@patch("dogesec_commons.objects.views.ArangoDBHelper.get_scos")
def test_sco_view_list_with_post_id(mock_get_scos):
    mock_get_scos.return_value = Response({"results": ["filtered-sco"]})
    request = factory.get("/api/objects/sco/?post_id=test123")
    response = SCOView.as_view({"get": "list"})(request)
    mock_get_scos.assert_called_once_with(matcher={"_obstracts_post_id": "test123"})
    assert response == mock_get_scos.return_value


@pytest.mark.django_db
@patch("dogesec_commons.objects.views.ArangoDBHelper.get_smos")
def test_smo_view_list_success(mock_get_smos):
    mock_get_smos.return_value = Response({"results": ["mock-smo"]})
    request = factory.get("/api/objects/smo/")
    response = SMOView.as_view({"get": "list"})(request)
    assert response == mock_get_smos.return_value
    mock_get_smos.assert_called_once()


@pytest.mark.django_db
@patch("dogesec_commons.objects.views.ArangoDBHelper.get_sros")
def test_sro_view_list_success(mock_get_sros):
    mock_get_sros.return_value = Response({"results": ["mock-sro"]})
    request = factory.get("/api/objects/sro/")
    response = SROView.as_view({"get": "list"})(request)
    assert response == mock_get_sros.return_value
    mock_get_sros.assert_called_once()


@pytest.mark.django_db
@patch("dogesec_commons.objects.views.ArangoDBHelper.get_sdos")
def test_sdo_view_list(mock_get_sdos):
    mock_get_sdos.return_value = Response({"results": ["filtered-sdo"]})
    request = factory.get("/api/objects/sdo/")
    response = SDOView.as_view({"get": "list"})(request)
    mock_get_sdos.assert_called_once()
    assert response == mock_get_sdos.return_value


class SingleObjectsViewTest(URLPatternsTestCase):
    router = routers.SimpleRouter()
    router.register('', ObjectsWithReportsView, 'object-view')
    urlpatterns = [
        path('objects/', include(router.urls)),
    ]
    stix_id = 'stix-object--'+str(uuid.uuid4())

    @patch("dogesec_commons.objects.views.ArangoDBHelper.get_objects_by_id")
    def test_retrieve(self, mock_get_objects_by_id):
        mock_get_objects_by_id.return_value = Response()
        url = f'/objects/{self.stix_id}/'
        response = self.client.get(url, format='json')
        mock_get_objects_by_id.assert_called_once_with(self.stix_id)
        assert response == mock_get_objects_by_id.return_value

    def test_lookup_value_regex(self):
        url = f'/objects/bad-stix-id/'
        response = self.client.get(url, format='json')
        assert response.status_code == 404, "should fail because of bad stix_id"


    @patch("dogesec_commons.objects.views.ArangoDBHelper.get_object_bundle")
    def test_bundle(self, mock_get_object_bundle):
        mock_get_object_bundle.return_value = Response()
        url = f'/objects/{self.stix_id}/bundle/'
        response = self.client.get(url, format='json')
        mock_get_object_bundle.assert_called_once_with(self.stix_id)
        assert response == mock_get_object_bundle.return_value

    

    @patch("dogesec_commons.objects.views.ArangoDBHelper.get_containing_reports")
    def test_reports(self, mock_get_containing_reports):
        mock_get_containing_reports.return_value = Response()
        url = f'/objects/{self.stix_id}/reports/'
        response = self.client.get(url, format='json')
        mock_get_containing_reports.assert_called_once_with(self.stix_id)
        assert response == mock_get_containing_reports.return_value

    

    @patch("dogesec_commons.objects.views.ArangoDBHelper.delete_report_object")
    def test_destroy_report(self, mock_delete_report_object):
        mock_delete_report_object.return_value = Response()
        report_id = f'report--{uuid.uuid4()}'
        url = f'/objects/{self.stix_id}/reports/{report_id}/'
        response = self.client.delete(url, format='json')
        assert response.status_code != 404
        mock_delete_report_object.assert_called_once_with(report_id=report_id, object_id=self.stix_id)
        assert response == mock_delete_report_object.return_value

    


# Generated by Django 5.1.5 on 2025-02-19 15:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='ai_content_check_variable',
            field=models.CharField(default=None, max_length=256, null=True),
        ),
    ]



import logging
from rest_framework import serializers

from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import gettext_lazy as _

from drf_spectacular.types import OpenApiTypes
from drf_spectacular.utils import extend_schema_field



class RelatedObjectField(serializers.RelatedField):
    lookup_key = 'pk'
    default_error_messages = {
        'required': _('This field is required.'),
        'does_not_exist': _('Invalid {lookup_key} "{lookup_value}" - object does not exist.'),
        'incorrect_type': _('Incorrect type. Expected valid {lookup_key} value, received "{lookup_value}", type: {data_type}.'),
    }
    def __init__(self, /, serializer, use_raw_value=False, **kwargs):
        self.internal_serializer: serializers.Serializer = serializer
        self.use_raw_value = use_raw_value
        super().__init__(**kwargs)

    def to_internal_value(self, data):
        try:
            instance = self.get_queryset().get(**{self.lookup_key: data})
            if self.use_raw_value:
                return data
            return instance
        except ObjectDoesNotExist as e:
            self.fail('does_not_exist', lookup_value=data, lookup_key=self.lookup_key)
        except BaseException as e:
            logging.exception(e)
            self.fail('incorrect_type', data_type=type(data), lookup_value=data, lookup_key=self.lookup_key)
        
    def to_representation(self, value):
        return self.internal_serializer.to_representation(value)


@extend_schema_field(OpenApiTypes.ANY)
class AnyField(serializers.Field):
    pass

class CommonErrorSerializer(serializers.Serializer):
    message = serializers.CharField(required=False)
    code    = serializers.IntegerField(required=True)
    details = serializers.JSONField(required=False)


from django.conf import settings
from arango.client import ArangoClient


def pytest_sessionstart(session):
    client = ArangoClient(hosts=settings.ARANGODB_HOST_URL)
    sys_db = client.db(
        "_system",
        username=settings.ARANGODB_USERNAME,
        password=settings.ARANGODB_PASSWORD,
    )
    if not sys_db.has_database(settings.ARANGODB_DATABASE):
        sys_db.create_database(settings.ARANGODB_DATABASE)



import contextlib
import logging
import typing, re
from arango import ArangoClient
from django.conf import settings
from rest_framework.response import Response
from drf_spectacular.utils import OpenApiParameter
from ..utils.pagination import Pagination
from rest_framework.exceptions import ValidationError, NotFound
from stix2arango.services import ArangoDBService
from . import conf

from django.http import HttpResponse
from django.conf import settings
from rest_framework import decorators, response, status


SDO_TYPES = set(
    [
        "attack-pattern",
        "campaign",
        "course-of-action",
        "grouping",
        "identity",
        "incident",
        "indicator",
        "infrastructure",
        "intrusion-set",
        "location",
        "malware",
        "malware-analysis",
        "note",
        "observed-data",
        "opinion",
        "report",
        "threat-actor",
        "sighting",
        "tool",
        "vulnerability",
        "weakness",
    ]
)

SCO_TYPES = set(
    [
        "artifact",
        "autonomous-system",
        "bank-account",
        "bank-card",
        "cryptocurrency-transaction",
        "cryptocurrency-wallet",
        "directory",
        "domain-name",
        "email-addr",
        "email-message",
        "file",
        "ipv4-addr",
        "ipv6-addr",
        "mac-addr",
        "mutex",
        "network-traffic",
        "phone-number",
        "process",
        "software",
        "url",
        "user-account",
        "user-agent",
        "windows-registry-key",
        "x509-certificate",
    ]
)
SDO_SORT_FIELDS = [
    "name_ascending",
    "name_descending",
    "created_ascending",
    "created_descending",
    "modified_ascending",
    "modified_descending",
    "type_ascending",
    "type_descending",
]
SRO_SORT_FIELDS = [
    "created_ascending",
    "created_descending",
    "modified_ascending",
    "modified_descending",
]


SCO_SORT_FIELDS = ["type_ascending", "type_descending"]


SMO_SORT_FIELDS = [
    "created_ascending",
    "created_descending",
    "type_ascending",
    "type_descending",
]


SMO_TYPES = set(
    [
        "marking-definition",
        "extension-definition",
        "language-content",
    ]
)

OBJECT_TYPES = SDO_TYPES.union(SCO_TYPES).union(["relationship"]).union(SMO_TYPES)


def positive_int(integer_string, cutoff=None, default=1):
    """
    Cast a string to a strictly positive integer.
    """
    with contextlib.suppress(ValueError, TypeError):
        ret = int(integer_string)
        if ret <= 0:
            return default
        if cutoff:
            return min(ret, cutoff)
        return ret
    return default


class ArangoDBHelper:
    max_page_size = conf.MAXIMUM_PAGE_SIZE
    page_size = conf.DEFAULT_PAGE_SIZE
    SRO_OBJECTS_ONLY_LATEST = getattr(settings, "SRO_OBJECTS_ONLY_LATEST", True)
    STIX_OBJECT_SCHEMA = {
        "type": "object",
        "properties": {
            "type": {
                "example": "domain-name",
            },
            "id": {
                "example": "domain-name--a86627d4-285b-5358-b332-4e33f3ec1075",
            },
        },
        "additionalProperties": True,
    }

    @staticmethod
    def get_like_literal(str: str):
        return str.replace("_", "\\_").replace("%", "\\%")

    @classmethod
    def like_string(cls, string: str):
        return "%" + cls.get_like_literal(string) + "%"

    def get_sort_stmt(self, sort_options: list[str], customs={}, doc_name="doc"):
        finder = re.compile(r"(.+)_((a|de)sc)ending")
        sort_field = self.query.get("sort", sort_options[0])
        if sort_field not in sort_options:
            return ""
        if m := finder.match(sort_field):
            field = m.group(1)
            direction = m.group(2).upper()
            if cfield := customs.get(field):
                return f"SORT {cfield} {direction}"
            return f"SORT {doc_name}.{field} {direction}"

    def query_as_array(self, key):
        query = self.query.get(key)
        if not query:
            return []
        return query.split(",")

    def query_as_bool(self, key, default=True):
        query_str = self.query.get(key)
        if not query_str:
            return default
        return query_str.lower() in ["true", "yes", "1", "y"]

    @classmethod
    def get_page_params(cls, kwargs):
        page_number = positive_int(kwargs.get("page"))
        page_limit = positive_int(
            kwargs.get("page_size"),
            cutoff=ArangoDBHelper.max_page_size,
            default=ArangoDBHelper.page_size,
        )
        return page_number, page_limit

    @classmethod
    def get_paginated_response(
        cls, data, page_number, page_size=page_size, full_count=0, result_key="objects"
    ):
        return Response(
            {
                "page_size": page_size or cls.page_size,
                "page_number": page_number,
                "page_results_count": len(data),
                "total_results_count": full_count,
                result_key: list(data),
            }
        )

    @classmethod
    def get_paginated_response_schema(cls, result_key="objects", schema=None):

        return {
            200: {
                "type": "object",
                "required": ["page_results_count", result_key],
                "properties": {
                    "page_size": {
                        "type": "integer",
                        "example": cls.max_page_size,
                    },
                    "page_number": {
                        "type": "integer",
                        "example": 3,
                    },
                    "page_results_count": {
                        "type": "integer",
                        "example": cls.page_size,
                    },
                    "total_results_count": {
                        "type": "integer",
                        "example": cls.page_size * cls.max_page_size,
                    },
                    result_key: {
                        "type": "array",
                        "items": schema or cls.STIX_OBJECT_SCHEMA,
                    },
                },
            },
            400: {
                "type": "object",
                "properties": {
                    "detail": {"type": "string"},
                    "code": {"type": "integer"},
                },
                "required": [
                    "code",
                ],
            },
        }

    @classmethod
    def get_schema_operation_parameters(self):
        parameters = [
            OpenApiParameter(
                "page",
                type=int,
                description=Pagination.page_query_description,
            ),
            OpenApiParameter(
                "page_size",
                type=int,
                description=Pagination.page_size_query_description,
            ),
        ]
        return parameters

    client = ArangoClient(hosts=settings.ARANGODB_HOST_URL)
    DB_NAME = conf.DB_NAME

    def __init__(self, collection, request, result_key="objects") -> None:
        self.collection = collection
        self.db = self.client.db(
            self.DB_NAME,
            username=settings.ARANGODB_USERNAME,
            password=settings.ARANGODB_PASSWORD,
        )
        self.result_key = result_key
        self.request = request
        self.query = request.query_params.dict() if request else dict()
        self.page, self.count = self.get_page_params(self.query)

    def execute_query(self, query, bind_vars={}, paginate=True):
        if paginate:
            bind_vars["offset"], bind_vars["count"] = self.get_offset_and_count(
                self.count, self.page
            )
        try:
            cursor = self.db.aql.execute(
                query, bind_vars=bind_vars, count=True, full_count=True
            )
        except Exception as e:
            logging.exception(e)
            raise ValidationError("aql: cannot process request")
        if paginate:
            return self.get_paginated_response(
                cursor,
                self.page,
                self.count,
                cursor.statistics()["fullCount"],
                result_key=self.result_key,
            )
        return list(cursor)

    def get_offset_and_count(self, count, page) -> tuple[int, int]:
        page = page or 1
        if page >= 2**32:
            raise ValidationError(f"invalid page `{page}`")
        offset = (page - 1) * count
        return offset, count

    def get_scos(self, matcher={}):
        types = SCO_TYPES
        other_filters = []

        if new_types := self.query_as_array("types"):
            types = types.intersection(new_types)
        bind_vars = {
            "@collection": self.collection,
            "types": list(types),
        }
        if value := self.query.get("value"):
            bind_vars["search_value"] = value.lower()
            other_filters.append(
                """
                (
                    doc.type == 'artifact' AND CONTAINS(LOWER(doc.payload_bin), @search_value) OR
                    doc.type == 'autonomous-system' AND CONTAINS(LOWER(doc.number), @search_value) OR
                    doc.type == 'bank-account' AND CONTAINS(LOWER(doc.iban_number), @search_value) OR
                    doc.type == 'bank-card' AND CONTAINS(LOWER(doc.number), @search_value) OR
                    doc.type == 'cryptocurrency-transaction' AND CONTAINS(LOWER(doc.hash), @search_value) OR
                    doc.type == 'cryptocurrency-wallet' AND CONTAINS(LOWER(doc.hash), @search_value) OR
                    doc.type == 'directory' AND CONTAINS(LOWER(doc.path), @search_value) OR
                    doc.type == 'domain-name' AND CONTAINS(LOWER(doc.value), @search_value) OR
                    doc.type == 'email-addr' AND CONTAINS(LOWER(doc.value), @search_value) OR
                    doc.type == 'email-message' AND CONTAINS(LOWER(doc.body), @search_value) OR
                    doc.type == 'file' AND CONTAINS(LOWER(doc.name), @search_value) OR
                    doc.type == 'ipv4-addr' AND CONTAINS(LOWER(doc.value), @search_value) OR
                    doc.type == 'ipv6-addr' AND CONTAINS(LOWER(doc.value), @search_value) OR
                    doc.type == 'mac-addr' AND CONTAINS(LOWER(doc.value), @search_value) OR
                    doc.type == 'mutex' AND CONTAINS(LOWER(doc.value), @search_value) OR
                    doc.type == 'network-traffic' AND CONTAINS(LOWER(doc.protocols), @search_value) OR
                    doc.type == 'phone-number' AND CONTAINS(LOWER(doc.number), @search_value) OR
                    doc.type == 'process' AND CONTAINS(LOWER(doc.pid), @search_value) OR
                    doc.type == 'software' AND CONTAINS(LOWER(doc.name), @search_value) OR
                    doc.type == 'url' AND CONTAINS(LOWER(doc.value), @search_value) OR
                    doc.type == 'user-account' AND CONTAINS(LOWER(doc.display_name), @search_value) OR
                    doc.type == 'user-agent' AND CONTAINS(LOWER(doc.string), @search_value) OR
                    doc.type == 'windows-registry-key' AND CONTAINS(LOWER(doc.key), @search_value) OR
                    doc.type == 'x509-certificate' AND CONTAINS(LOWER(doc.subject), @search_value)
                    //generic
                    OR
                    CONTAINS(LOWER(doc.value), @search_value) OR
                    CONTAINS(LOWER(doc.name), @search_value) OR
                    CONTAINS(LOWER(doc.number), @search_value)
                )
                """.strip()
            )

        if matcher:
            bind_vars["matcher"] = matcher
            other_filters.insert(0, "MATCHES(doc, @matcher)")

        if other_filters:
            other_filters = "FILTER " + " AND ".join(other_filters)

        query = f"""
            FOR doc in @@collection SEARCH doc.type IN @types AND doc._is_latest == TRUE
            {other_filters or ""}
            {self.get_sort_stmt(SCO_SORT_FIELDS)}

            COLLECT id = doc.id INTO docs
            LET doc = FIRST(FOR d in docs[*].doc SORT d.modified OR d.created DESC, d._record_modified DESC RETURN d)
            
            LIMIT @offset, @count
            RETURN KEEP(doc, KEYS(doc, true))
        """
        return self.execute_query(query, bind_vars=bind_vars)

    def get_smos(self):
        types = SMO_TYPES
        if new_types := self.query_as_array("types"):
            types = types.intersection(new_types)
        bind_vars = {
            "@collection": self.collection,
            "types": list(types),
        }
        other_filters = {}
        query = f"""
            FOR doc in @@collection
            SEARCH doc.type IN @types AND doc._is_latest == TRUE
            {other_filters or ""}
            {self.get_sort_stmt(SMO_SORT_FIELDS)}


            COLLECT id = doc.id INTO docs
            LET doc = FIRST(FOR d in docs[*].doc SORT d.modified OR d.created DESC, d._record_modified DESC RETURN d)

            LIMIT @offset, @count
            RETURN  KEEP(doc, KEYS(doc, true))
        """
        return self.execute_query(query, bind_vars=bind_vars)

    def get_sdos(self):
        types = SDO_TYPES
        if new_types := self.query_as_array("types"):
            types = types.intersection(new_types)

        bind_vars = {
            "@collection": self.collection,
            "types": list(types),
        }
        other_filters = []
        search_filters = ["doc._is_latest == TRUE"]
        if term := self.query.get("labels"):
            bind_vars["labels"] = term
            other_filters.append("doc.labels[? ANY FILTER CONTAINS(CURRENT, @labels)]")

        if term := self.query.get("name"):
            bind_vars["name"] = "%" + self.get_like_literal(term) + "%"
            search_filters.append("doc.name LIKE @name")

        if q := self.query.get("visible_to"):
            bind_vars["visible_to"] = q
            bind_vars["marking_visible_to_all"] = (
                "marking-definition--bab4a63c-aed9-4cf5-a766-dfca5abac2bb",
                "marking-definition--94868c89-83c2-464b-929b-a1a8aa3c8487",
            )
            search_filters.append(
                "(doc.created_by_ref IN [@visible_to, NULL] OR @marking_visible_to_all ANY IN doc.object_marking_refs)"
            )

        if other_filters:
            other_filters = "FILTER " + " AND ".join(other_filters)

        query = f"""
            FOR doc in @@collection
            SEARCH doc.type IN @types AND {' AND '.join(search_filters)}
            {other_filters or ""}
            {self.get_sort_stmt(SDO_SORT_FIELDS)}

            
            COLLECT id = doc.id INTO docs
            LET doc = FIRST(FOR d in docs[*].doc SORT d.modified OR d.created DESC, d._record_modified DESC RETURN d)

            LIMIT @offset, @count
            RETURN  KEEP(doc, KEYS(doc, true))
        """
        # return HttpResponse(f"{query}\n\n// {__import__('json').dumps(bind_vars)}")
        return self.execute_query(query, bind_vars=bind_vars)

    def get_objects_by_id(self, id):
        bind_vars = {
            "@view": self.collection,
            "id": id,
        }
        query = """
            FOR doc in @@view
            SEARCH doc.id == @id AND doc._is_latest == TRUE
            LIMIT 1
            RETURN KEEP(doc, KEYS(doc, true))
        """
        objs = self.execute_query(query, bind_vars=bind_vars, paginate=False)
        if not objs:
            raise NotFound("No object with id")
        return Response(objs[0])

    def get_object_bundle(self, stix_id):
        bind_vars = {
            "@view": self.collection,
            "id": stix_id,
        }
        rel_search_filters = []
        search_filters = []
        if not self.query_as_bool("include_embedded_refs", True):
            rel_search_filters.append("doc._is_ref != TRUE")

        if types := self.query_as_array("types"):
            rel_search_filters.append(
                "(doc._target_type IN @types OR doc._source_type IN @types)"
            )
            search_filters.append("FILTER doc.type IN @types")
            bind_vars["types"] = types

        visible_to_filter = ""
        if q := self.query.get("visible_to"):
            bind_vars["visible_to"] = q
            bind_vars["marking_visible_to_all"] = (
                "marking-definition--bab4a63c-aed9-4cf5-a766-dfca5abac2bb",
                "marking-definition--94868c89-83c2-464b-929b-a1a8aa3c8487",
            )
            visible_to_filter = "FILTER doc.created_by_ref == @visible_to OR @marking_visible_to_all ANY IN doc.object_marking_refs OR doc.created_by_ref == NULL"

        query = """
            LET bundle_ids = FLATTEN(FOR doc in @@view SEARCH (doc.source_ref == @id or doc.target_ref == @id) AND doc._is_latest == TRUE /* rel_search_extras */ RETURN [doc._id, doc._from, doc._to])
            FOR doc IN @@view
            SEARCH (doc._id IN bundle_ids OR (doc.id == @id AND doc._is_latest == TRUE))
            // extra_search
            // visible_to_filter
            LIMIT @offset, @count
            RETURN KEEP(doc, KEYS(doc, TRUE))
        """
        if rel_search_filters:
            query = query.replace(
                "/* rel_search_extras */", " AND " + " AND ".join(rel_search_filters)
            )
        if search_filters:
            query = query.replace("// extra_search", "\n".join(search_filters))

        if visible_to_filter:
            query = query.replace("// visible_to_filter", visible_to_filter)
        return self.execute_query(query, bind_vars=bind_vars)

    def get_containing_reports(self, id):
        bind_vars = {
            "@view": self.collection,
            "id": id,
        }
        query = """
            LET report_ids = (
                FOR doc in @@view
                FILTER doc.id == @id
                RETURN DISTINCT doc._stixify_report_id
            )
            FOR report in @@view
            SEARCH report.type == 'report' AND report.id IN report_ids
            LIMIT @offset, @count
            RETURN KEEP(report, KEYS(report, TRUE))
        """
        return self.execute_query(query, bind_vars=bind_vars)

    def get_sros(self):
        bind_vars = {
            "@collection": self.collection,
        }

        search_filters = ["doc._is_latest == TRUE"]

        if terms := self.query_as_array("source_ref_type"):
            bind_vars["source_ref_type"] = terms
            search_filters.append("doc._source_type IN @source_ref_type")

        if terms := self.query_as_array("target_ref_type"):
            bind_vars["target_ref_type"] = terms
            search_filters.append("doc._target_type IN @target_ref_type")

        if term := self.query.get("relationship_type"):
            bind_vars["relationship_type"] = "%" + self.get_like_literal(term) + "%"
            search_filters.append("doc.relationship_type LIKE @relationship_type")

        if not self.query_as_bool("include_embedded_refs", True):
            search_filters.append("doc._is_ref != TRUE")

        if term := self.query.get("target_ref"):
            bind_vars["target_ref"] = term
            search_filters.append("doc.target_ref == @target_ref")

        if term := self.query.get("source_ref"):
            bind_vars["source_ref"] = term
            search_filters.append("doc.source_ref == @source_ref")

        if not self.SRO_OBJECTS_ONLY_LATEST:
            search_filters[0] = (
                "(doc._is_latest == TRUE OR doc._target_type IN @sco_types OR doc._source_type IN @sco_types)"
            )
            bind_vars["sco_types"] = list(SCO_TYPES)

        if q := self.query.get("visible_to"):
            bind_vars["visible_to"] = q
            bind_vars["marking_visible_to_all"] = (
                "marking-definition--bab4a63c-aed9-4cf5-a766-dfca5abac2bb",
                "marking-definition--94868c89-83c2-464b-929b-a1a8aa3c8487",
            )
            search_filters.append(
                "(doc.created_by_ref IN [@visible_to, NULL] OR @marking_visible_to_all ANY IN doc.object_marking_refs)"
            )

        query = f"""
            FOR doc in @@collection
            SEARCH doc.type == 'relationship' AND { ' AND '.join(search_filters) }
            {self.get_sort_stmt(SRO_SORT_FIELDS)}

            COLLECT id = doc.id INTO docs
            LET doc = FIRST(FOR d in docs[*].doc SORT d.modified OR d.created DESC, d._record_modified DESC RETURN d)

            LIMIT @offset, @count
            RETURN KEEP(doc, KEYS(doc, true))

        """
        # return HttpResponse(content=f"{query}\n\n// {__import__('json').dumps(bind_vars)}")
        return self.execute_query(query, bind_vars=bind_vars)

    def delete_report_object(self, report_id, object_id):
        db_service = ArangoDBService(
            self.DB_NAME,
            [],
            [],
            create=False,
            create_db=False,
            create_collection=False,
            username=settings.ARANGODB_USERNAME,
            password=settings.ARANGODB_PASSWORD,
            host_url=settings.ARANGODB_HOST_URL,
        )
        query = """
        let doc_ids = (
            FOR doc IN @@view
            SEARCH doc.id IN [@object_id, @report_id] AND doc._stixify_report_id == @report_id
            SORT doc.object_refs
            RETURN [doc._id, doc.id]
        )
        LET doc_id = FIRST(doc_ids[* FILTER CURRENT[1]== @object_id])
        LET report_id = FIRST(FIRST(doc_ids[* FILTER CURRENT[1] == @report_id]))

        RETURN [report_id, doc_id, (FOR d IN APPEND([doc_id], (
                FOR doc IN @@view
                SEARCH doc._from == doc_id[0] OR doc._to == doc_id[0]
                RETURN [doc._id, doc.id])
            )
        FILTER d != NULL
        RETURN d)]
        """
        bind_vars = {
            "@view": self.collection,
            "object_id": object_id,
            "report_id": report_id,
        }
        report_idkey, doc_id, ids_to_be_removed = self.execute_query(
            query, bind_vars=bind_vars, paginate=False
        )[0]
        # separate into collections
        collections = {}
        bind_vars = {"ckeys": {}}
        queries = []
        stix_ids = []
        for key_id, stix_id in ids_to_be_removed:
            stix_ids.append(stix_id)
            try:
                db_service.db.delete_document(key_id)
            except Exception as e:
                logging.exception("failed to delete object %s", key_id)

        if not stix_ids:
            return response.Response(status=status.HTTP_204_NO_CONTENT)
        resp = self.execute_query(
            """
            FOR doc in @@collection FILTER doc._id == @report_idkey
                UPDATE {_key: doc._key} WITH {object_refs: REMOVE_VALUES(doc.object_refs, @stix_ids)} IN @@collection
                RETURN {new_length: LENGTH(NEW.object_refs), old_length: LENGTH(doc.object_refs)}
                """,
            bind_vars={
                "report_idkey": report_idkey,
                "stix_ids": stix_ids,
                "@collection": report_idkey.split("/")[0],
            },
            paginate=False,
        )
        logging.info(
            f"removed references from report.object_refs: {resp} // {ids_to_be_removed}"
        )
        if doc_id:
            doc_collection_name = doc_id[0].split("/")[0]
            db_service.update_is_latest_several_chunked(
                list(set([object_id] + stix_ids)),
                doc_collection_name,
                doc_collection_name.removesuffix("_vertex_collection").removesuffix(
                    "_edge_collection"
                )
                + "_edge_collection",
            )
        return response.Response(status=status.HTTP_204_NO_CONTENT)
